        ;*****************************************************************
        ;*                       DISPLAY.FUN                             *
        ;* A group of 16C64 functions to run the LM044L Hitachi 4 x 20   *
        ;* display.                                                      *
        ;*       S.E.Weigold    -   Pole Zero Corporation  -  20 May 94  *
        ;*****************************************************************
        
        ;DISCLR FUNCTION: Clears display sets DDRAM address to home
        ; and returns cursor to home ( address 0 ).

        DISCLR  CLRF    PORTE   
                MOVLW   1
                MOVWF   PORTB           ;CLEAR DISPLAY AND GOTO HOME
                CALL    PULSEE          ;STROBE DISPLAY
                CALL    CKBUSY          ;CHECK FOR INSTRUCTION COMPLETION
                RETURN    

        ;*****
        
        ;FSET FUNCTION:   Function set sets interface data length, number
        ;of display lines, and character font.

        FSET    CLRF    PORTE    
                MOVLW   38
                MOVWF   PORTB           ;FUNCTION SET DL L F
                CALL    PULSEE          ;STROBE DISPLAY          
                CALL    CKBUSY
                RETURN    

        ;*****
        
        ;ONDIS FUNCTION; turns on display, and cursor blink.
        
        ONDIS   CLRF    PORTE   
                MOVLW   0x0C            ;TURN ON DISPLAY
                MOVWF   PORTB                              
                CALL    PULSEE          ;STROBE DISPLAY
                CALL    CKBUSY          ;CHECK FOR INSTRUCTION COMPLETION
                RETURN    

        ;*****
        
        ;ESET FUNCTION: Entry mode set, Sets cursor move direction
        ; and display shift.

        ESET    CLRF    PORTE   
                MOVLW   06              ;ENTRY MODE SET
                MOVWF   PORTB                              
                CALL    PULSEE          ;STROBE DISPLAY
                CALL    CKBUSY          ;CHECK FOR INSTRUCTION COMPLETION
                RETURN    
        
        ;*****
        
        ;SETDD FUNCTION: Sets displat DDRAM addres to value set in W 
        ;register prior to function call. Performs move cursor function

        SETDD   CLRF    PORTE    
                MOVWF   PORTB           ;MOVE TO PORT
                CALL    PULSEE          ;STROBE DISPLAY
                CALL    CKBUSY          ;CHECK FOR INSTRUCTION COMPLETION
                RETURN    
        
        ;*****
        
        ;WDATA FUNCTION: Writes charater specified in W prior to function
        ;call to current DDRAM address in display.

        WDATA   CLRF    PORTE    
                BSF     PORTE,RS
                MOVWF   PORTB           ;WRITE CHAR DATA FROM W TO DISPLAY
                CALL    PULSEE          ;STROBE DISPLAY
                CALL    CKBUSY          ;CHECK FOR INSTRUCTION COMPLETION
                RETURN    

        ;*****

        ;CKBUSY FUNCTION: Reads address counter, and busy flag from display
        ;to determine instruction completion.  does not return until complete.
        
        CKBUSY  BSF     STATUS,RP0      ;SET PAGE ONE
                MOVLW   0xFF
                MOVWF   DDRB            ;CONFIGURE PORTB AS INPUTS
                BCF     STATUS,RP0      ;SET PAGE ZERO

                BCF     PORTE,RS
                BSF     PORTE,RW        ;SELECT READ OPERATIONS
                BSF     PORTE,E         ;ENABLE DISPLAY
                CALL    SHORT           ;DELAY
                MOVF    PORTB,0         ;READ DATA
                BCF     PORTE,E         ;DISABLE DISPLAY
                MOVWF   SDAT            ;STORE PORTB DATA
                BTFSC   SDAT,7          ;CHECK BUSY FLAG
                GOTO    CKBUSY          ;IF BUSY, TRY AGAIN
                
                BSF     STATUS,RP0      ;SET PAGE ONE
                CLRF    DDRB            ;SET PORTB TO OUTPUTS
                BCF     STATUS,RP0      ;SET PAGE ZERO
                RETURN                  ;ELSE RETURN

        ;*****

        ;DISCFG ROUTINE: Configures LM044l Display, and prepares it to 
        ;accept data.
        
        DISCFG  CALL    DISCLR          ;CLEAR DISPLAY
                CALL    FSET            ;FUNCTION SET
                CALL    ONDIS           ;DISPLAY ON
                CALL    ESET            ;ENTRY MODE SET
                RETURN

        ;*****

        ;PULSEE ROUTINE; Generates strobe pulse for LM044L display

        PULSEE  CALL    SHORT           ;DELAY
                BSF     PORTE,E         ;RISING CLOCK EDGE
                CALL    SHORT
                BCF     PORTE,E         ;FALLING CLOCK EDGE
                RETURN

        ;*****

        ;LINE ROUTINES; Places cursor at beginning of indicated line.
        
        LINE1   MOVLW   0x80
                CALL    SETDD           ;SET NEW DISPLAY ADDRESS
                RETURN

        ;*****

        LINE2   MOVLW   0xC0
                CALL    SETDD           ;SET NEW DISPLAY ADDRESS   
                RETURN

        ;*****

        LINE3   MOVLW   0x94
                CALL    SETDD           ;SET NEW DISPLAY ADDRESS   
                RETURN

        ;*****

        LINE4   MOVLW   0xD4
                CALL    SETDD           ;SET NEW DISPLAY ADDRESS   
                RETURN

        ;*****

